@@ -10,11 +10,12 @@ use std::ops::Deref;
 use std::ptr::copy_nonoverlapping;
 use std::{cmp, result};
 
+use vhost_user_backend::bitmap::BitmapMmapRegion;
 use virtio_queue::DescriptorChain;
 use vm_memory::bitmap::{Bitmap, BitmapSlice};
 use vm_memory::{
-    Address, ByteValued, GuestMemory, GuestMemoryError, GuestMemoryMmap, GuestMemoryRegion, Le16,
-    Le32, Le64, VolatileMemory, VolatileMemoryError, VolatileSlice,
+    Address, ByteValued, GuestMemory, GuestMemoryError, GuestMemoryMmap, GuestMemoryRegion,
+    VolatileMemory, VolatileMemoryError, VolatileSlice,
 };
 
 use crate::file_traits::FileReadWriteAtVolatile;
@@ -184,7 +185,7 @@ impl<'a, B: BitmapSlice> DescriptorChainConsumer<'a, B> {
 /// Reader will skip iterating over descriptor chain when first writable
 /// descriptor is encountered.
 #[derive(Clone)]
-pub struct Reader<'a, B = ()> {
+pub struct Reader<'a, B = BitmapMmapRegion> {
     buffer: DescriptorChainConsumer<'a, B>,
 }
 
@@ -247,11 +248,12 @@ impl<'a, B: Bitmap + BitmapSlice + 'static> Reader<'a, B> {
         Ok(unsafe { obj.assume_init() })
     }
 
-    /// Reads data from the descriptor chain buffer into a File at offset `off`.
-    /// Returns the number of bytes read from the descriptor chain buffer.
-    /// The number of bytes read can be less than `count` if there isn't
-    /// enough data in the descriptor chain buffer.
-    pub fn read_to_at<F: FileReadWriteAtVolatile<B>>(
+    /// Write data from the descriptor chain buffer into a File at offset `off`.
+    ///
+    /// Return the number of bytes written.  This can be less than `count` if there isn't enough
+    /// data in the descriptor chain buffer, or if the file’s write function performed a short
+    /// write.
+    pub fn write_to_file_at<F: FileReadWriteAtVolatile<B>>(
         &mut self,
         dst: F,
         count: usize,
@@ -283,7 +285,7 @@ impl<'a, B: Bitmap + BitmapSlice + 'static> Reader<'a, B> {
     }
 }
 
-impl<'a, B: BitmapSlice> io::Read for Reader<'a, B> {
+impl<B: BitmapSlice> io::Read for Reader<'_, B> {
     fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
         self.buffer.consume(buf.len(), |bufs| {
             let mut rem = buf;
@@ -291,9 +293,12 @@ impl<'a, B: BitmapSlice> io::Read for Reader<'a, B> {
             for vs in bufs {
                 let copy_len = cmp::min(rem.len(), vs.len());
 
-                // Safe because we have already verified that `vs` points to valid memory.
+                // SAFETY: Safe because we verify that we do not read outside
+                // of the slice's bound. The slice guard will only get dropped
+                // after the function returns. This will keep the pointer valid
+                // while reads are happening.
                 unsafe {
-                    copy_nonoverlapping(vs.as_ptr() as *const u8, rem.as_mut_ptr(), copy_len);
+                    copy_nonoverlapping(vs.ptr_guard().as_ptr(), rem.as_mut_ptr(), copy_len);
                 }
                 rem = &mut rem[copy_len..];
                 total += copy_len;
@@ -311,7 +316,7 @@ impl<'a, B: BitmapSlice> io::Read for Reader<'a, B> {
 /// Writer will start iterating the descriptors from the first writable one and will
 /// assume that all following descriptors are writable.
 #[derive(Clone)]
-pub struct Writer<'a, B = ()> {
+pub struct Writer<'a, B = BitmapMmapRegion> {
     buffer: DescriptorChainConsumer<'a, B>,
 }
 
@@ -369,11 +374,11 @@ impl<'a, B: Bitmap + BitmapSlice + 'static> Writer<'a, B> {
         self.buffer.available_bytes()
     }
 
-    /// Writes data to the descriptor chain buffer from a File at offset `off`.
-    /// Returns the number of bytes written to the descriptor chain buffer.
-    /// The number of bytes written can be less than `count` if
-    /// there isn't enough data in the descriptor chain buffer.
-    pub fn write_from_at<F: FileReadWriteAtVolatile<B>>(
+    /// Read data into the descriptor chain buffer from a File at offset `off`.
+    ///
+    /// Return the number of bytes read.  This can be less than `count` if there isn't enough data
+    /// in the descriptor chain buffer, or if the file’s read function performed a short read.
+    pub fn read_from_file_at<F: FileReadWriteAtVolatile<B>>(
         &mut self,
         src: F,
         count: usize,
@@ -397,7 +402,7 @@ impl<'a, B: Bitmap + BitmapSlice + 'static> Writer<'a, B> {
     }
 }
 
-impl<'a, B: BitmapSlice> io::Write for Writer<'a, B> {
+impl<B: BitmapSlice> io::Write for Writer<'_, B> {
     fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
         self.buffer.consume(buf.len(), |bufs| {
             let mut rem = buf;
@@ -405,9 +410,12 @@ impl<'a, B: BitmapSlice> io::Write for Writer<'a, B> {
             for vs in bufs {
                 let copy_len = cmp::min(rem.len(), vs.len());
 
-                // Safe because we have already verified that `vs` points to valid memory.
+                // SAFETY: Safe because we ensure that we do not write over the
+                // slice's bounds. The slice guard will only get dropped after
+                // the function returns. This will keep the pointer valid while
+                // writes are happening.
                 unsafe {
-                    copy_nonoverlapping(rem.as_ptr(), vs.as_ptr(), copy_len);
+                    copy_nonoverlapping(rem.as_ptr(), vs.ptr_guard_mut().as_ptr(), copy_len);
                 }
                 vs.bitmap().mark_dirty(0, copy_len);
                 rem = &rem[copy_len..];
@@ -429,39 +437,39 @@ pub enum DescriptorType {
     Writable,
 }
 
-#[derive(Copy, Clone, Debug, Default)]
-#[repr(C)]
-struct virtq_desc {
-    addr: Le64,
-    len: Le32,
-    flags: Le16,
-    next: Le16,
-}
-
-// Safe because it only has data and has no implicit padding.
-unsafe impl ByteValued for virtq_desc {}
-
-#[derive(Copy, Clone, Debug, Default)]
-#[repr(C)]
-struct virtq_avail {
-    flags: Le16,
-    idx: Le16,
-    ring: Le16,
-}
-
-// Safe because it only has data and has no implicit padding.
-unsafe impl ByteValued for virtq_avail {}
-
 #[cfg(test)]
 mod tests {
     use super::*;
     use virtio_queue::{Queue, QueueOwnedT, QueueT};
-    use vm_memory::{Bytes, GuestAddress};
+    use vm_memory::{Bytes, GuestAddress, Le16, Le32, Le64};
 
     const VIRTQ_DESC_F_NEXT: u16 = 0x1;
     const VIRTQ_DESC_F_WRITE: u16 = 0x2;
     const MAX_QUEUE_SIZE: u16 = 32768;
 
+    #[derive(Copy, Clone, Debug, Default)]
+    #[repr(C)]
+    struct virtq_desc {
+        addr: Le64,
+        len: Le32,
+        flags: Le16,
+        next: Le16,
+    }
+
+    // Safe because it only has data and has no implicit padding.
+    unsafe impl ByteValued for virtq_desc {}
+
+    #[derive(Copy, Clone, Debug, Default)]
+    #[repr(C)]
+    struct virtq_avail {
+        flags: Le16,
+        idx: Le16,
+        ring: Le16,
+    }
+
+    // Safe because it only has data and has no implicit padding.
+    unsafe impl ByteValued for virtq_avail {}
+
     /// Test utility function to create a descriptor chain in guest memory.
     pub fn create_descriptor_chain(
         memory: &GuestMemoryMmap,
