@@ -12,7 +12,7 @@ use std::path::Path;
 use std::str::FromStr;
 use std::sync::Arc;
 use std::time::Duration;
-use std::{env, process};
+use std::{cmp, env, process};
 use virtiofsd::idmap::{GidMap, UidMap};
 
 use clap::{CommandFactory, Parser};
@@ -33,6 +33,23 @@ use virtiofsd::vhost_user::{Error, VhostUserFsBackendBuilder, MAX_TAG_LEN};
 use virtiofsd::{limits, oslib, soft_idmap};
 use vm_memory::{GuestMemoryAtomic, GuestMemoryMmap};
 
+/// Maximum number of memory areas (slots) supported by the vhost-user-backend crate.
+///
+/// The constant has the same name there, but is not exported.
+const MAX_MEM_SLOTS: u64 = 509;
+
+/// How many file descriptors to reserve for internal use.
+///
+/// The exact value has been chosen mostly arbitrarily, but we know we need one FD per shared
+/// memory area, which is where the `MAX_MEM_SLOTS` comes from.
+///
+/// Given how allocating FD towards the guest quota works, we also need to add one FD per thread in
+/// our pool: FDs are created first, and only then accounted for.  All threads must be able to
+/// simultaneously create an FD and then have it be accounted for, so we need to make room for as
+/// many additional FDs as there are threads in the pool.  The thread pool size is set at runtime,
+/// though, so cannot be taken into account here, but instead where this constant is used.
+const INTERNAL_FD_RESERVE: u64 = MAX_MEM_SLOTS + 100;
+
 type Result<T> = std::result::Result<T, Error>;
 
 fn parse_seccomp(src: &str) -> std::result::Result<SeccompAction, &'static str> {
@@ -254,7 +271,7 @@ struct Opt {
     #[arg(short = 'f')]
     compat_foreground: bool,
 
-    /// Enable security label support. Expects SELinux xattr on file creation
+    /// Enable security label support (implies --xattr). Expects SELinux xattr on file creation
     /// from client and stores it in the newly created file.
     #[arg(long = "security-label")]
     security_label: bool,
@@ -349,10 +366,10 @@ struct Opt {
     /// different ways:
     ///
     /// - abort: Whenever any error occurs, return a hard error to the vhost-user front-end (e.g.
-    ///          QEMU), aborting migration.
+    ///   QEMU), aborting migration.
     ///
     /// - guest-error: Let migration finish, but the guest will be unable to access any of the
-    ///                affected inodes, receiving only errors.
+    ///   affected inodes, receiving only errors.
     ///
     /// This parameter is ignored on the source side.
     #[arg(long = "migration-on-error", default_value = "abort")]
@@ -514,7 +531,7 @@ fn initialize_logging(opt: &Opt) {
     if opt.syslog {
         if let Err(e) = syslog::init(syslog::Facility::LOG_USER, log_level, None) {
             set_default_logger(log_level);
-            warn!("can't enable syslog: {}", e);
+            warn!("can't enable syslog: {e}");
         }
     } else {
         set_default_logger(log_level);
@@ -530,7 +547,7 @@ fn set_signal_handlers() {
     let signals = vec![libc::SIGHUP, libc::SIGTERM];
     for s in signals {
         if let Err(e) = signal::register_signal_handler(s, handle_signal) {
-            error!("Setting signal handlers: {}", e);
+            error!("Setting signal handlers: {e}");
             process::exit(1);
         }
     }
@@ -552,14 +569,11 @@ fn parse_modcaps(
             let (action, cap_name) = modcap.split_at(1);
             let cap_name = cap_name.to_uppercase();
             if !matches!(action, "+" | "-") {
-                error!(
-                    "invalid modcap action: expecting '+'|'-' but found '{}'",
-                    action
-                );
+                error!("invalid modcap action: expecting '+'|'-' but found '{action}'");
                 process::exit(1);
             }
             if let Err(error) = capng::name_to_capability(&cap_name) {
-                error!("invalid capability '{}': {}", &cap_name, error);
+                error!("invalid capability '{cap_name}': {error}");
                 process::exit(1);
             }
 
@@ -595,10 +609,7 @@ fn drop_capabilities(inode_file_handles: InodeFileHandlesMode, modcaps: Option<S
     if inode_file_handles != InodeFileHandlesMode::Never {
         let required_cap = "DAC_READ_SEARCH".to_owned();
         if disabled_caps.contains(&required_cap) {
-            error!(
-                "can't disable {} when using --inode-file-handles={:?}",
-                &required_cap, inode_file_handles
-            );
+            error!("can't disable {required_cap} when using --inode-file-handles={inode_file_handles:?}");
             process::exit(1);
         }
         required_caps.insert(required_cap);
@@ -612,11 +623,11 @@ fn drop_capabilities(inode_file_handles: InodeFileHandlesMode, modcaps: Option<S
         capng::Type::PERMITTED | capng::Type::EFFECTIVE,
         required_caps.iter().map(String::as_str).collect(),
     ) {
-        error!("can't set up the child capabilities: {}", e);
+        error!("can't set up the child capabilities: {e}");
         process::exit(1);
     }
     if let Err(e) = capng::apply(capng::Set::BOTH) {
-        error!("can't apply the child capabilities: {}", e);
+        error!("can't apply the child capabilities: {e}");
         process::exit(1);
     }
 }
@@ -684,7 +695,7 @@ fn main() {
     }
 
     let xattrmap = opt.xattrmap.clone();
-    let xattr = xattrmap.is_some() || opt.posix_acl || opt.xattr;
+    let xattr = xattrmap.is_some() || opt.posix_acl || opt.security_label || opt.xattr;
     let thread_pool_size = opt.thread_pool_size;
     let readdirplus = match opt.cache {
         CachePolicy::Never => false,
@@ -738,12 +749,12 @@ fn main() {
             let pid_file_name = socket.to_owned() + ".pid";
             let pid_file_path = Path::new(pid_file_name.as_str());
             let pid_file = write_pid_file(pid_file_path).unwrap_or_else(|error| {
-                error!("Error creating pid file '{}': {}", pid_file_name, error);
+                error!("Error creating pid file '{pid_file_name}': {error}");
                 process::exit(1);
             });
 
             let listener = Listener::new(socket, true).unwrap_or_else(|error| {
-                error!("Error creating listener: {}", error);
+                error!("Error creating listener: {error}");
                 process::exit(1);
             });
 
@@ -771,11 +782,25 @@ fn main() {
         }
     }
 
-    limits::setup_rlimit_nofile(opt.rlimit_nofile).unwrap_or_else(|error| {
-        error!("Error increasing number of open files: {}", error);
+    let fd_count_limit = limits::setup_rlimit_nofile(opt.rlimit_nofile).unwrap_or_else(|error| {
+        error!("Error increasing number of open files: {error}");
         process::exit(1)
     });
 
+    // Account for guest FDs that are created first and only accounted for then (see doc comment on
+    // `INTERNAL_FD_RESERVE`
+    let internal_fd_reserve = INTERNAL_FD_RESERVE + cmp::max(opt.thread_pool_size as u64, 1);
+    let guest_fd_limit = fd_count_limit.checked_sub(internal_fd_reserve).unwrap_or_else(|| {
+        error!("Maximum number of file descriptors too small: Limit is {fd_count_limit}, must be at least {internal_fd_reserve}");
+        process::exit(1)
+    });
+
+    // Warn the user if there is a suspiciously low limit on the guest FD count that will make it
+    // hard to actually do something; the number of `128` is completely arbitrary.
+    if guest_fd_limit < 128 {
+        warn!("File descriptor count limit is very small, leaving only {guest_fd_limit} file descriptors for the guest");
+    }
+
     let mut sandbox = Sandbox::new(
         shared_dir.to_string(),
         opt.sandbox,
@@ -783,14 +808,14 @@ fn main() {
         opt.gid_map,
     )
     .unwrap_or_else(|error| {
-        error!("Error creating sandbox: {}", error);
+        error!("Error creating sandbox: {error}");
         process::exit(1)
     });
 
     // Enter the sandbox, from this point the process will be isolated (or not)
     // as chosen in '--sandbox'.
     let listener = sandbox.enter(listener).unwrap_or_else(|error| {
-        error!("Error entering sandbox: {}", error);
+        error!("Error entering sandbox: {error}");
         process::exit(1)
     });
 
@@ -820,6 +845,7 @@ fn main() {
         migration_mode: opt.migration_mode,
         uid_map: Some(opt.translate_uid),
         gid_map: Some(opt.translate_gid),
+        guest_fd_limit,
         ..Default::default()
     };
 
@@ -864,7 +890,7 @@ fn run_generic_fs<F: FileSystem + SerializableFileSystem + Send + Sync + 'static
             .set_tag(tag)
             .build(fs)
             .unwrap_or_else(|error| {
-                error!("Error creating vhost-user backend: {}", error);
+                error!("Error creating vhost-user backend: {error}");
                 process::exit(1)
             }),
     );
@@ -879,7 +905,7 @@ fn run_generic_fs<F: FileSystem + SerializableFileSystem + Send + Sync + 'static
     info!("Waiting for vhost-user socket connection...");
 
     if let Err(e) = daemon.start(listener) {
-        error!("Failed to start daemon: {:?}", e);
+        error!("Failed to start daemon: {e:?}");
         process::exit(1);
     }
 
@@ -888,7 +914,7 @@ fn run_generic_fs<F: FileSystem + SerializableFileSystem + Send + Sync + 'static
     if let Err(e) = daemon.wait() {
         match e {
             HandleRequest(Disconnected) => info!("Client disconnected, shutting down"),
-            _ => error!("Waiting for daemon failed: {:?}", e),
+            _ => error!("Waiting for daemon failed: {e:?}"),
         }
     }
 }
