@@ -132,7 +132,7 @@ impl<'a> InodeData {
     /// Get an `O_PATH` file for this inode
     pub fn get_file(&'a self) -> io::Result<InodeFile<'a>> {
         match &self.file_or_handle {
-            FileOrHandle::File(f) => Ok(InodeFile::Ref(f)),
+            FileOrHandle::File(f) => Ok(InodeFile::Ref(f.get_file())),
             FileOrHandle::Handle(h) => {
                 let file = h.open(libc::O_PATH)?;
                 Ok(InodeFile::Owned(file))
@@ -206,8 +206,8 @@ impl<'a> InodeData {
                 _ => "unknown inode type",
             };
             format!(
-                "[{}; mount_id={} device_id={} inode_id={}]",
-                mode, self.ids.mnt_id, self.ids.dev, self.ids.ino,
+                "[{mode}; mount_id={} device_id={} inode_id={}]",
+                self.ids.mnt_id, self.ids.dev, self.ids.ino,
             )
         }
     }
